/*
 * Written by Professor Bill Tomlinson for Fall 2005 Programming for Interactivity class at UCI.
 * wmt@uci.edu
 */

import java.applet.*;
import java.awt.*;

public class BasicGraphicsSystem extends Applet implements Runnable {
	//how many blocks in x
	public static int xSize = 5;
	//how many blocks in y
	public static int ySize = 5;
	//how many pixels per block
	public static int cellSize = 100;
	public static double[][][] blocks = new double[xSize][ySize][3];
	Color backgroundColor = Color.gray;
	Canvas gridCanvas;
	
	//This method is called once, when the applet is first run
	public void run() {
		//sets all the blocks to be gray at the beginning
		for (int i = 0; i <xSize; i++){
			for (int j = 0; j <ySize; j++){
				setBrightness(i,j,0.5);
			}
		}

		//call the update method repeatedly
		int counter = 0;
		while (true) {
			update(counter);
			counter++;
		}
	}
	
	//use this method to change the colors of the blocks
	private void update(int counter) {
		System.out.println("doing step #"+counter);

		//Assignment 1: insert code below this line
		
		for (int i = 0; i < xSize; i++){
			for (int j = 0; j < ySize; j++){	
				setBrightness(i, j, (Math.sin((double)(counter+i)/5)+1)/2);			
			}
		}

		//Assignment 1: insert code above this line.

		gridCanvas.repaint();
		try {
			Thread.sleep(50);
		} catch (Exception e) {}
	}

	//initialize the grid world
	public void init() {
		System.out.println("initting");
		//setBasicGraphicsSystem(this);
		setLayout(new BorderLayout());
		gridCanvas = new Canvas() {
			public void paint(Graphics g) {
				setBackground(backgroundColor);
				g.clearRect(1, 1, (xSize * cellSize) - 2, (ySize * cellSize) - 2);

				for (int thisX = 0; thisX < xSize; thisX++) {
					for (int thisY = 0; thisY < ySize; thisY++) {
						g.setColor(new Color((float)blocks[thisX][thisY][0], (float)blocks[thisX][thisY][1],(float)blocks[thisX][thisY][2]));
						g.fillRect(thisX * cellSize+1, thisY * cellSize+1, cellSize - 1, cellSize - 1);
					}
				}
			}
			//begin double buffering code
			private Image offscreen;
			public void update(Graphics g)
			{
			    offscreen = createImage( xSize*cellSize,ySize*cellSize );
			    Graphics gg = offscreen.getGraphics();
			    gg.setColor( getBackground() );
			    gg.fillRect( 0, 0, xSize*cellSize, ySize*cellSize);
			    gg.setColor( getForeground() );
			    paint( gg );
			    gg.dispose();
			    g.drawImage( offscreen, 0, 0, null );
			}
		};
		Panel controlPanel = new Panel();
		controlPanel.setSize(30, 10);
		gridCanvas.setSize(500, 500);
		this.setLayout(new BorderLayout());
		add(BorderLayout.CENTER, gridCanvas);
		add(BorderLayout.SOUTH, controlPanel);
		setVisible(true);
		validate();
		gridCanvas.repaint();
		this.run();
	} // init

	public void paint(Graphics g) {
		gridCanvas.repaint();
	}
	
	/*
	 * Convenience function to let you set a grayscale value to a block
	 */
	private void setBrightness(int x, int y, double brightness) {
		setColor(x, y, brightness, brightness, brightness);
	}

	/*
	 * Convenience function to let you set r, g, and b separately
	 */
	private void setColor(int x, int y, double r, double g, double b) {
		r = clamp(r, 0, 1);
		g = clamp(g, 0, 1);
		b = clamp(b, 0, 1);
		blocks[x][y][0]= r;
		blocks[x][y][1]= g;
		blocks[x][y][2]= b;
	}
	/*
	 * Convenience function to let you clamp a value
	 */
	private double clamp(double num, int low, int high) {
		if (num < low) num = low;
		if (num > high) num = high;
		return num;
	}
	/*
	 * Convenience function to return the color of a block, as an array of doubles
	 */
	private double[] getColor(int x, int y) {
		return blocks[x][y];
	}
	/*
	 * Convenience function to return the brightness of a block, as an array of doubles
	 */
	private double getBrightness(int x, int y) {
		return (blocks[x][y][0]+blocks[x][y][1]+blocks[x][y][2])/3;
	}		
}